// -----------------------------------------------------------------------------
// talent_single_target.nss
// -----------------------------------------------------------------------------
/*
    Generic Ability Script for single target abilities
*/
// -----------------------------------------------------------------------------
// georg / petert
// -----------------------------------------------------------------------------

#include "log_h"
#include "abi_templates"
#include "sys_traps_h"
#include "spell_constants_h"

void _HandleImpact(struct EventSpellScriptImpactStruct stEvent)
{
    // -------------------------------------------------------------------------
    // VAR BLOCK
    // -------------------------------------------------------------------------
    int    nScalingVector   = SCALING_VECTOR_DURATION;
    int    nAttackingValue  = PROPERTY_ATTRIBUTE_SPELLPOWER;
    int    nResistance      = RESISTANCE_INVALID;
    float  fDuration        = 0.0f;
    float  fScaledValue     = 0.0f;
    int    nEffect          = 0;
    int    nHandler         = SPELL_HANDLER_CUSTOM;
    effect eDamage;
    effect eEffect;
    effect[] eEffects;

    // make sure there is a location, just in case
    if (IsObjectValid(stEvent.oTarget) == TRUE)
    {
        stEvent.lTarget = GetLocation(stEvent.oTarget);
    }

    // -------------------------------------------------------------------------
    // Handle Spells
    // -------------------------------------------------------------------------
    switch (stEvent.nAbility)
    {
        case 10204:
        {
            // remove stacking effects
            RemoveStackingEffects(stEvent.oTarget, stEvent.oCaster, stEvent.nAbility);

            float fDamage = (100.0f + GetCreatureSpellPower(stEvent.oCaster)) * FIREBALL_DAMAGE_FACTOR;
            effect eDamage = EffectDamage(fDamage, DAMAGE_TYPE_FIRE, DAMAGE_EFFECT_FLAG_NONE);
            ApplyEffectOnObject(EFFECT_DURATION_TYPE_INSTANT, eDamage, stEvent.oTarget, 0.0, stEvent.oCaster, stEvent.nAbility);
            ApplyEffectDamageOverTime(stEvent.oTarget, stEvent.oCaster, stEvent.nAbility, fDamage, PRIDE_DEMON_FIRE_BOLT_DOT_DURATION, DAMAGE_TYPE_FIRE, 10);
            break;
        }
        case 11007:
        {
            Ability_ApplyObjectImpactVFX(stEvent.nAbility, stEvent.oTarget);
            float fDamage = (100.0f + GetCreatureSpellPower(stEvent.oCaster)) * CONE_OF_COLD_DAMAGE_FACTOR;
            effect eDamage = EffectDamage(fDamage, DAMAGE_TYPE_COLD, DAMAGE_EFFECT_FLAG_NONE);
            ApplyEffectOnObject(EFFECT_DURATION_TYPE_INSTANT, eDamage, stEvent.oTarget, 0.0, stEvent.oCaster, stEvent.nAbility);

            // physical resistance
            if (ResistanceCheck(stEvent.oCaster, stEvent.oTarget, PROPERTY_ATTRIBUTE_STRENGTH, RESISTANCE_PHYSICAL) == FALSE)
            {
                float fDuration = GetRankAdjustedEffectDuration(stEvent.oTarget, CONE_OF_COLD_DURATION);

                // remove stacking effects
                RemoveStackingEffects(stEvent.oTarget, stEvent.oCaster, stEvent.nAbility);

                effect eEffect = EffectParalyze(PRIDE_DEMON_FROST_BOLT_VFX);
                ApplyEffectOnObject(EFFECT_DURATION_TYPE_TEMPORARY, eEffect, stEvent.oTarget, fDuration, stEvent.oCaster, stEvent.nAbility);
            }

            break;
        }
    }
}

void main()
{
    event ev = GetCurrentEvent();
    int nEventType = GetEventType(ev);

    switch(nEventType)
    {
        case EVENT_TYPE_SPELLSCRIPT_PENDING:
        {
            struct EventSpellScriptPendingStruct stEvent = Events_GetEventSpellScriptPendingParameters(ev);

            Ability_SetSpellscriptPendingEventResult(COMMAND_RESULT_SUCCESS);

            break;
        }

        case EVENT_TYPE_SPELLSCRIPT_CAST:
        {
            // Get a structure with the event parameters
            struct EventSpellScriptCastStruct stEvent = Events_GetEventSpellScriptCastParameters(ev);

            // determine if attack hits
            object oMainWeapon = GetItemInEquipSlot(INVENTORY_SLOT_MAIN);
            struct CombatAttackResultStruct stAttack = Combat_PerformAttack(stEvent.oCaster, stEvent.oTarget, oMainWeapon,0.0f,stEvent.nAbility);

            // Hand this through to cast_impact
            SetAbilityResult(stEvent.oCaster, stEvent.nResistanceCheckResult);

            break;
        }

        case EVENT_TYPE_SPELLSCRIPT_IMPACT:
        {
            // Get a structure with the event parameters
            struct EventSpellScriptImpactStruct stEvent = Events_GetEventSpellScriptImpactParameters(ev);

            Log_Trace(LOG_CHANNEL_COMBAT_ABILITY, GetCurrentScriptName() + ".EVENT_TYPE_SPELLSCRIPT_IMPACT",Log_GetAbilityNameById(stEvent.nAbility));

            // Handle impact
            _HandleImpact(stEvent);

            SendEventOnCastAt(stEvent.oTarget,stEvent.oCaster, stEvent.nAbility, TRUE);

            break;
        }
        case EVENT_TYPE_AOE_HEARTBEAT:
        {
            int nAbility = GetEventInteger(ev,0);
            object oCreator = GetEventCreator(ev);
            float fInterval = TEMPEST_INTERVAL;
            int bHandled = FALSE;
            Log_Trace(LOG_CHANNEL_EVENTS, GetCurrentScriptName(), "Ability " + ToString(nAbility) + " Heartbeat");
            object[] oTargets = GetCreaturesInAOE(OBJECT_SELF);

            // run through all creatures in AoE
            int nCount = 0;
            effect eEffect;
            int nMax = GetArraySize(oTargets);
            for (nCount = 0; nCount < nMax; nCount++)
            {
                if (CheckSpellResistance(oTargets[nCount], oCreator, nAbility) == FALSE)
                {
                    float fDamage = (100.0f + GetCreatureSpellPower(oCreator)) * TEMPEST_DAMAGE_FACTOR;
        //          ApplyEffectDamageOverTime(oTarget, oCreator, nAbility, fDamage, INFERNO_INTERVAL, DAMAGE_TYPE_ELECTRICITY);
                    eEffect = EffectDamage(fDamage, DAMAGE_TYPE_ELECTRICITY);
                    ApplyEffectOnObject(EFFECT_DURATION_TYPE_INSTANT, eEffect, oTargets[nCount], 0.0f, oCreator, nAbility);
                } else
                {
                    UI_DisplayMessage(oTargets[nCount], UI_MESSAGE_RESISTED);
                }
            }

            if (fInterval > 0.0f)
            {
                if (IsObjectValid(OBJECT_SELF) && (IsModalAbilityActive(oCreator,nAbility) || !Ability_IsModalAbility(nAbility)))
                {
                    // signal next heartbeat
                    DelayEvent(fInterval + 0.05f, OBJECT_SELF, ev);
                }
            }

        }
    }
}