// -----------------------------------------------------------------------------
// spell_modalsingletarget.nss
// -----------------------------------------------------------------------------
/*
    Generic Ability Script for modal single target abilities
*/
// -----------------------------------------------------------------------------
// PeterT
// -----------------------------------------------------------------------------

#include "log_h"
#include "abi_templates"
#include "spell_constants_h"
#include "plt_tut_modal"

// add effects
void _ActivateModalAbility(struct EventSpellScriptImpactStruct stEvent)
{
    // effects
    effect[] eEffects;
    int bPartywide = FALSE;
    int bEffectValid = TRUE;

    if(IsFollower(stEvent.oCaster))
    {
        WR_SetPlotFlag(PLT_TUT_MODAL, TUT_MODAL_1, TRUE);
    }

    if (IsObjectValid(stEvent.oTarget) == TRUE)
    {
        stEvent.lTarget = GetLocation(stEvent.oTarget);
    }

    // -------------------------------------------------------------------------
    // Handle Spells
    // -------------------------------------------------------------------------

    switch (stEvent.nAbility)
    {
        case 93004:
        {
            // determine power of earth weapons
            float fPower = MaxF(1.0,MinF((GetCreatureSpellPower(stEvent.oCaster) / 15.0f), 15.0f));
            eEffects[0] = EffectEnchantment(ENCHANTMENT_CROW_POISON, FloatToInt(fPower));
            bPartywide = TRUE;

            break;
        }
        case 93000:
        {
            // determine power of frost weapons
            float fPower = MaxF(1.0,MinF((GetCreatureSpellPower(stEvent.oCaster) / 7.5f), 10.0f));
            eEffects[0] = EffectEnchantment(ENCHANTMENT_COLD, FloatToInt(fPower));
            bPartywide = TRUE;

            break;
        }
        case 93001:
        {
            // determine power of fire weapons
            float fPower = MaxF(1.0,MinF((GetCreatureSpellPower(stEvent.oCaster) / 7.5f), 10.0f));
            eEffects[0] = EffectEnchantment(ENCHANTMENT_FIRE, FloatToInt(fPower));
            bPartywide = TRUE;

            break;
        }
        case 93002:
        {
            // determine power of lightning weapons
            float fPower = MaxF(1.0,MinF((GetCreatureSpellPower(stEvent.oCaster) / 7.5f), 10.0f));
            eEffects[0] = EffectEnchantment(ENCHANTMENT_LIGHTNING, FloatToInt(fPower));
            bPartywide = TRUE;
            break;
        }
        case 93003:
        {
            // determine power of spirit weapons
            float fPower = MaxF(1.0,MinF((GetCreatureSpellPower(stEvent.oCaster) / 7.5f), 20.0f));
            eEffects[0] = EffectEnchantment(ENCHANTMENT_HOLY, FloatToInt(fPower));
            bPartywide = TRUE;
            break;
        }
    }

    if (bEffectValid == TRUE)
    {
        Log_Trace_Spell("_ActivateModalAbility", "Activating modal ability.", stEvent.nAbility, OBJECT_INVALID);

        Ability_ApplyUpkeepEffects(stEvent.oCaster, stEvent.nAbility, eEffects, stEvent.oTarget, bPartywide);

        Log_Trace_Spell("_ActivateModalAbility", "Modal ability activated.", stEvent.nAbility, OBJECT_INVALID);
    }
    else
    {
        Log_Trace_Spell("_ActivateModalAbility", "Invalid Effect. We are not activating modal ability.", stEvent.nAbility, OBJECT_INVALID);
    }

}

// remove effects
void _DeactivateModalAbility(object oCaster, int nAbility)
{
    Log_Trace_Spell("_DeactivateModalAbility", "Deactivate modal ability.", nAbility, OBJECT_INVALID);

    // remove effects
    Effects_RemoveUpkeepEffect(oCaster, nAbility);
}

void main()
{
    event ev = GetCurrentEvent();
    int nEventType = GetEventType(ev);

    switch(nEventType)
    {
        case EVENT_TYPE_SPELLSCRIPT_PENDING:
        {
            // hand through
            Ability_SetSpellscriptPendingEventResult(COMMAND_RESULT_SUCCESS);

            break;
        }

        case EVENT_TYPE_SPELLSCRIPT_CAST:
        {
            // Get a structure with the event parameters
            struct EventSpellScriptCastStruct stEvent = Events_GetEventSpellScriptCastParameters(ev);

            // we just hand this through to cast_impact
            SetAbilityResult(stEvent.oCaster, stEvent.nResistanceCheckResult);


            break;
        }

        case EVENT_TYPE_SPELLSCRIPT_IMPACT:
        {
            // Get a structure with the event parameters
            struct EventSpellScriptImpactStruct stEvent = Events_GetEventSpellScriptImpactParameters(ev);

            Log_Trace(LOG_CHANNEL_COMBAT_ABILITY, GetCurrentScriptName() + ".EVENT_TYPE_SPELLSCRIPT_IMPACT",Log_GetAbilityNameById(stEvent.nAbility));

            // Remove any previously existing effects from same spellid to avoid stacking
            Ability_PreventAbilityEffectStacking(stEvent.oTarget, stEvent.oCaster, stEvent.nAbility);

            // activate ability
            _ActivateModalAbility(stEvent);

            break;
        }

        case EVENT_TYPE_SPELLSCRIPT_DEACTIVATE:
        {
            // Get a structure with the event parameters
            struct EventSpellScriptDeactivateStruct stEvent = Events_GetEventSpellScriptDeactivateParameters(ev);

            // is ability active?
            if (IsModalAbilityActive(stEvent.oCaster, stEvent.nAbility) == TRUE)
            {
                _DeactivateModalAbility(stEvent.oCaster, stEvent.nAbility);
            }

            // Setting Return Value (abort means we aborted the ability)
            Ability_SetSpellscriptPendingEventResult(COMMAND_RESULT_INVALID);

            break;
        }
    }
}